
surface.CreateFont( "coolvetica", 19, 500, true, false, "ScoreboardPlayerName" )
surface.CreateFont( "coolvetica", 22, 500, true, false, "ScoreboardPlayerNameBig" )

local texGradient = surface.GetTextureID( "gui/center_gradient" )

local texRatings = {}
texRatings[ 'none' ] 		= surface.GetTextureID( "gui/silkicons/user" )
texRatings[ 'smile' ] 		= surface.GetTextureID( "gui/silkicons/emoticon_smile" )
texRatings[ 'bad' ] 		= surface.GetTextureID( "gui/silkicons/exclamation" )
texRatings[ 'love' ] 		= surface.GetTextureID( "gui/silkicons/heart" )
texRatings[ 'artistic' ] 	= surface.GetTextureID( "gui/silkicons/palette" )
texRatings[ 'star' ] 		= surface.GetTextureID( "gui/silkicons/star" )
texRatings[ 'builder' ] 	= surface.GetTextureID( "gui/silkicons/wrench" )

surface.GetTextureID( "gui/silkicons/emoticon_smile" )
local PANEL = {}

/*---------------------------------------------------------
   Name: Init
---------------------------------------------------------*/
function PANEL:Init()
	self.Size = 36
	self:OpenInfo( false )

	self.infoCard	= vgui.Create( "qHUDScorePlayerInfoCard", self )

	self.lblName 		= vgui.Create( "Label", self )
	self.lblFrags 		= vgui.Create( "Label", self )
	self.lblDeaths 		= vgui.Create( "Label", self )
	self.lblPing 		= vgui.Create( "Label", self )
	self.lblLoss 		= vgui.Create( "Label", self )

	// If you don't do this it'll block your clicks
	self.lblName:SetMouseInputEnabled( false )
	self.lblFrags:SetMouseInputEnabled( false )
	self.lblDeaths:SetMouseInputEnabled( false )
	self.lblPing:SetMouseInputEnabled( false )
	self.lblLoss:SetMouseInputEnabled( false )

	self.imgAvatar = vgui.Create( "AvatarImage", self )

	self:SetCursor( "hand" )
end

/*---------------------------------------------------------
   Name: Paint
---------------------------------------------------------*/
function PANEL:Paint()
	local color = GAMEMODE:GetTeamColor( self.Player )

	if ( !ValidEntity( self.Player ) ) then return end

	if ( self.Player:GetFriendStatus() == "friend" ) then
		starcolor = Color( 255, 255, 255, 255 ) 
	else
		starcolor = Color( 255, 255, 255, 0 ) 
	end

	if ( self.Open || self.Size != self.TargetSize ) then
		DrawBox( 0, 16, self:GetWide(), self:GetTall() - 16, color, 1 )
		DrawBox( 2, 16, self:GetWide() - 4, self:GetTall() - 16 - 2, Color( 250, 250, 245, 255 ), 1 )
		
		surface.SetTexture( texGradient )
		surface.SetDrawColor( 255, 255, 255, 255 )
		surface.DrawTexturedRect( 2, 16, self:GetWide() - 4, self:GetTall() - 16 - 2 ) 
	end

	DrawBox( 0, 0, self:GetWide(), 36, color, 1 )

	surface.SetTexture( texGradient )
	if ( self.Player == LocalPlayer() ) then
		surface.SetDrawColor( 255, 255, 255, 150 + math.sin( RealTime() * 2 ) * 50 )
	else
		surface.SetDrawColor( 255, 255, 255, 70 )
	end
	surface.DrawTexturedRect( 0, 0, self:GetWide(), 36 ) 

	// This should be an image panel!
	surface.SetTexture( self.texRating )
	surface.SetDrawColor( 255, 255, 255, 255 )
	surface.DrawTexturedRect( self:GetWide() - 16 - 8, 36 / 2 - 8, 16, 16 ) 

	surface.SetTexture( texRatings[ 'star' ] )
	surface.SetDrawColor( starcolor )
	surface.SetFont( "ScoreboardPlayerNameBig" )
	surface.DrawTexturedRectRotated( self.imgAvatar:GetWide() + surface.GetTextSize( self.Player:Nick() ) + 25, 36 / 2 - 8, 16, 16, 0 ) 

	return true
end

/*---------------------------------------------------------
   Name: UpdatePlayerData
---------------------------------------------------------*/
function PANEL:SetPlayer( ply )
	self.Player = ply

	self.infoCard:SetPlayer( ply )
	self.imgAvatar:SetPlayer( ply )

	self:UpdatePlayerData()
end

function PANEL:CheckRating( name, count )
	if ( self.Player:GetNetworkedInt( "Rating."..name, 0 ) > count ) then
		count = self.Player:GetNetworkedInt( "Rating."..name, 0 )
		self.texRating = texRatings[ name ]
	end

	return count
end

/*---------------------------------------------------------
   Name: UpdatePlayerData
---------------------------------------------------------*/
function PANEL:UpdatePlayerData()
	if ( !self.Player ) then return end
	if ( !self.Player:IsValid() ) then return end

	self.lblName:SetText( self.Player:Nick() )
	self.lblFrags:SetText( self.Player:Frags() )
	self.lblDeaths:SetText( self.Player:Deaths() )
	self.lblPing:SetText( self.Player:Ping() )
	self.lblLoss:SetText( self.Player:PacketLoss() )

	// Work out what icon to draw
	self.texRating = surface.GetTextureID( "gui/silkicons/emoticon_smile" )

	self.texRating = texRatings[ 'none' ]
	local count = 0

	count = self:CheckRating( 'smile', count )
	count = self:CheckRating( 'love', count )
	count = self:CheckRating( 'artistic', count )
	count = self:CheckRating( 'star', count )
	count = self:CheckRating( 'builder', count )
	count = self:CheckRating( 'bad', count )
end

/*---------------------------------------------------------
   Name: PerformLayout
---------------------------------------------------------*/
function PANEL:ApplySchemeSettings()
	self.lblName:SetFont( "ScoreboardPlayerNameBig" )
	self.lblFrags:SetFont( "ScoreboardPlayerName" )
	self.lblDeaths:SetFont( "ScoreboardPlayerName" )
	self.lblPing:SetFont( "ScoreboardPlayerName" )
	self.lblLoss:SetFont( "ScoreboardPlayerName" )

	self.lblName:SetFGColor( color_black )
	self.lblFrags:SetFGColor( color_black )
	self.lblDeaths:SetFGColor( color_black )
	self.lblPing:SetFGColor( color_black )
	self.lblLoss:SetFGColor( color_black )
end

/*---------------------------------------------------------
   Name: PerformLayout
---------------------------------------------------------*/
function PANEL:DoClick( x, y )
	if ( self.Open ) then
		surface.PlaySound( "ui/buttonclickrelease.wav" )
	else
		surface.PlaySound( "ui/buttonclick.wav" )
	end

	self:OpenInfo( !self.Open )
end

/*---------------------------------------------------------
   Name: PerformLayout
---------------------------------------------------------*/
function PANEL:OpenInfo( bool )
	if ( bool ) then
		self.TargetSize = 150
	else
		self.TargetSize = 36
	end

	self.Open = bool
end

/*---------------------------------------------------------
   Name: PerformLayout
---------------------------------------------------------*/
function PANEL:Think()
	if ( self.Size != self.TargetSize ) then
		self.Size = math.Approach( self.Size, self.TargetSize, ( math.abs( self.Size - self.TargetSize ) + 1 ) * 10 * FrameTime() )
		self:PerformLayout()
		SCOREBOARD:InvalidateLayout()
	end

	if ( !self.PlayerUpdate || self.PlayerUpdate < CurTime() ) then
		self.PlayerUpdate = CurTime() + 0.5
		self:UpdatePlayerData()
	end
end

/*---------------------------------------------------------
   Name: PerformLayout
---------------------------------------------------------*/
function PANEL:PerformLayout()
	self.imgAvatar:SetPos( 2, 2 )
	self.imgAvatar:SetSize( 32, 32 )

	self:SetSize( self:GetWide(), self.Size )

	self.lblName:SizeToContents()
	self.lblName:SetPos( 24, 2 )
	self.lblName:MoveRightOf( self.imgAvatar, 8 )

	local COLUMN_SIZE = 50

	self.lblLoss:SetPos( self:GetWide() - COLUMN_SIZE * 1, 0 )
	self.lblPing:SetPos( self:GetWide() - COLUMN_SIZE * 2, 0 )
	self.lblDeaths:SetPos( self:GetWide() - COLUMN_SIZE * 3, 0 )
	self.lblFrags:SetPos( self:GetWide() - COLUMN_SIZE * 4, 0 )

	if ( self.Open || self.Size != self.TargetSize ) then
		self.infoCard:SetVisible( true )
		self.infoCard:SetPos( 4, self.imgAvatar:GetTall() + 10 )
		self.infoCard:SetSize( self:GetWide() - 8, self:GetTall() - self.lblName:GetTall() - 10 )
	else
		self.infoCard:SetVisible( false )
	end
end

/*---------------------------------------------------------
   Name: PerformLayout
---------------------------------------------------------*/
function PANEL:HigherOrLower( row )
	if ( !self.Player:IsValid() || self.Player:Team() == TEAM_CONNECTING ) then return false end
	if ( !row.Player:IsValid() || row.Player:Team() == TEAM_CONNECTING ) then return true end

	if ( self.Player:Frags() == row.Player:Frags() ) then
		return self.Player:Deaths() < row.Player:Deaths()
	end

	return self.Player:Frags() > row.Player:Frags()
end

vgui.Register( "qHUDScorePlayerRow", PANEL, "Button" )

